{- $Id: MPI_CCalls.hs,v 1.1.1.2 2001/02/28 04:48:03 michaelw Exp $
 - -*- Haskell -*-
 -----------------------------------------------------------------------------
 - MPI_CCalls.hs - import MPI ccalls to low-level Haskell functions
 -----------------------------------------------------------------------------
 -
 - hMPI - a Haskell binding for MPI 1.1/1.2
 - Copyright (C) 2000,2001 Michael Weber <michael.weber@post.rwth-aachen.de>
 -
 - This library is free software; you can redistribute it and/or
 - modify it under the terms of the GNU Lesser General Public License
 - as published by the Free Software Foundation; either version 2 of
 - the License, or (at your option) any later version.
 - 
 - This library is distributed in the hope that it will be useful, but
 - WITHOUT ANY WARRANTY; without even the implied warranty of
 - MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 - Lesser General Public License for more details.
 - 
 - You should have received a copy of the GNU Lesser General Public
 - License along with this library; if not, write to the Free Software
 - Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 - USA
 - 
 -----------------------------------------------------------------------------
 - NOTES:                                                                      
 -   
 -----------------------------------------------------------------------------
 - TODO:                                                                       
 - * document this file properly                                               
 -----------------------------------------------------------------------------
 -}

module MPI_CCalls
    ( module MPI_CCalls   -- everything is exported
    )
where

import MPI_MarshalInternal
import Foreign.C
import Foreign.Ptr

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


-- Transport -----------------------------------------------------------------
foreign import ccall "MPI_Send" unsafe
	mpi_Send :: MPI_Buffer a -> CInt -> MPI_Datatype -> MPI_Rank -> MPI_Tag 
                 -> MPI_Comm -> IO MPI_ReturnCode

foreign import ccall "MPI_Recv" unsafe
	mpi_Recv :: MPI_Buffer a -> CInt -> MPI_Datatype -> MPI_Rank -> MPI_Tag 
                 -> MPI_Comm -> MPI_Status -> IO MPI_ReturnCode

foreign import ccall "MPI_Sendrecv" unsafe
	mpi_Sendrecv :: MPI_Buffer a -> CInt -> MPI_Datatype -> MPI_Rank 
                     -> MPI_Tag
                     -> MPI_Buffer a -> CInt -> MPI_Datatype -> MPI_Rank 
                     -> MPI_Tag 
                     -> MPI_Comm -> MPI_Status -> IO MPI_ReturnCode

foreign import ccall "MPI_Bcast" unsafe
	mpi_Bcast :: MPI_Buffer a -> CInt -> MPI_Datatype -> MPI_Rank -> MPI_Comm
                  -> IO MPI_ReturnCode

foreign import ccall "MPI_Isend" unsafe
	mpi_Isend :: MPI_Buffer a -> CInt -> MPI_Datatype -> MPI_Rank -> MPI_Tag
                  -> MPI_Comm -> MPI_Request -> IO MPI_ReturnCode

foreign import ccall "MPI_Irecv" unsafe
	mpi_Irecv :: MPI_Buffer a -> CInt -> MPI_Datatype -> MPI_Rank -> MPI_Tag
                  -> MPI_Comm -> MPI_Request -> IO MPI_ReturnCode

foreign import ccall "MPI_Wait" unsafe
        mpi_Wait :: MPI_Request -> MPI_Status -> IO MPI_ReturnCode

foreign import ccall "MPI_Test" unsafe
        mpi_Test :: MPI_Request -> Ptr CInt -> MPI_Status -> IO MPI_ReturnCode

foreign import ccall "MPI_Probe" unsafe
        mpi_Probe :: MPI_Rank -> MPI_Tag -> MPI_Comm -> MPI_Status 
                  -> IO MPI_ReturnCode

foreign import ccall "MPI_Iprobe" unsafe
        mpi_Iprobe :: MPI_Rank -> MPI_Tag -> MPI_Comm -> Ptr CInt -> MPI_Status 
                   -> IO MPI_ReturnCode

foreign import ccall "MPI_Cancel" unsafe
        mpi_Cancel :: MPI_Request -> IO MPI_ReturnCode

foreign import ccall "MPI_Test_cancelled" unsafe
        mpi_Test_cancelled :: MPI_Status -> Ptr CInt -> IO MPI_ReturnCode

foreign import ccall "MPI_Request_free" unsafe
        mpi_Request_free :: MPI_Request -> IO MPI_ReturnCode


#ifdef HAVE_MPI_REQUEST_GET_STATUS
foreign import ccall "MPI_Request_get_status" unsafe
        mpi_Request_get_status :: MPI_Request -> Ptr CInt -> MPI_Status
                               -> IO MPI_ReturnCode
#else
mpi_Request_get_status :: MPI_Request -> Ptr CInt -> MPI_Status
                       -> IO MPI_ReturnCode
mpi_Request_get_status _ _ _ = unsupportedCall "MPI_Request_get_status"
#endif


-- Reduction -----------------------------------------------------------------
foreign import ccall "MPI_Reduce" unsafe
	mpi_Reduce :: MPI_Buffer a -> MPI_Buffer a -> CInt -> MPI_Datatype 
                   -> MPI_Op -> MPI_Rank -> MPI_Comm -> IO MPI_ReturnCode


-- Management ----------------------------------------------------------------
-- -- for hMPI-internal use only
-- mpiUndefined :: Int
-- mpiUndefined = ``MPI_UNDEFINED''


foreign import ccall "hmpi_init" unsafe
	hmpi_init :: IO MPI_ReturnCode

foreign import ccall "MPI_Finalize" unsafe
	mpi_finalize :: IO MPI_ReturnCode

foreign import ccall "MPI_Initialized" unsafe
	mpi_Initialized :: Ptr CInt -> IO MPI_ReturnCode

#ifdef HAVE_MPI_FINALIZED
foreign import ccall "MPI_Finalized" unsafe
	mpi_Finalized :: Ptr CInt -> IO MPI_ReturnCode
#else
mpi_Finalized :: Ptr CInt -> IO MPI_ReturnCode
mpi_Finalized _ = unsupportedCall "MPI_Finalized"
#endif

foreign import ccall "MPI_Abort" unsafe
	mpi_Abort :: MPI_Comm -> MPI_Error -> IO MPI_ReturnCode

foreign import ccall "MPI_Get_version" unsafe
	mpi_Get_version :: Ptr CInt -> Ptr CInt -> IO MPI_ReturnCode

foreign import ccall "MPI_Get_processor_name" unsafe
	mpi_Get_processor_name :: Ptr Char -> Ptr CInt -> IO MPI_ReturnCode

foreign import ccall "MPI_Wtime" unsafe
	mpi_Wtime :: IO CDouble

foreign import ccall "MPI_Wtick" unsafe
	mpi_Wtick :: IO CDouble

{-
 - mpi_Get_count == hmpi_status_get_count
 -}

foreign import ccall "MPI_Get_elements" unsafe
	mpi_Get_elements :: MPI_Status -> MPI_Datatype -> Ptr CInt 
                         -> IO MPI_ReturnCode


-- Group ---------------------------------------------------------------------
foreign import ccall "MPI_Group_rank" unsafe
	mpi_Group_rank :: MPI_Group -> Ptr MPI_Rank -> IO MPI_ReturnCode

foreign import ccall "MPI_Group_size" unsafe
	mpi_Group_size :: MPI_Group -> Ptr CInt -> IO MPI_ReturnCode

foreign import ccall "MPI_Group_free" unsafe
	mpi_Group_free :: Ptr MPI_Group -> IO MPI_ReturnCode

foreign import ccall "MPI_Group_union" unsafe
	mpi_Group_union :: MPI_Group -> MPI_Group -> Ptr MPI_Group 
                        -> IO MPI_ReturnCode

foreign import ccall "MPI_Group_intersection" unsafe
	mpi_Group_intersection :: MPI_Group -> MPI_Group -> Ptr MPI_Group 
                               -> IO MPI_ReturnCode

foreign import ccall "MPI_Group_difference" unsafe
	mpi_Group_difference :: MPI_Group -> MPI_Group -> Ptr MPI_Group 
                             -> IO MPI_ReturnCode

foreign import ccall "MPI_Group_incl" unsafe
	mpi_Group_incl :: MPI_Group -> CInt -> Ptr MPI_Rank -> Ptr MPI_Group 
                       -> IO MPI_ReturnCode

foreign import ccall "MPI_Group_excl" unsafe
	mpi_Group_excl :: MPI_Group -> CInt -> Ptr MPI_Rank -> Ptr MPI_Group 
                       -> IO MPI_ReturnCode

foreign import ccall "MPI_Group_compare" unsafe
	mpi_Group_compare :: MPI_Group -> MPI_Group -> Ptr CInt 
                          -> IO MPI_ReturnCode


-- Comm ----------------------------------------------------------------------
foreign import ccall "MPI_Comm_rank" unsafe
	mpi_Comm_rank :: MPI_Comm -> Ptr CInt -> IO MPI_ReturnCode

foreign import ccall "MPI_Comm_size" unsafe
	mpi_Comm_size :: MPI_Comm -> Ptr CInt -> IO MPI_ReturnCode

foreign import ccall "MPI_Comm_create" unsafe
	mpi_Comm_create :: MPI_Comm -> MPI_Group -> Ptr MPI_Comm 
                        -> IO MPI_ReturnCode

foreign import ccall "MPI_Comm_free" unsafe
	mpi_Comm_free :: Ptr MPI_Comm -> IO MPI_ReturnCode

foreign import ccall "MPI_Comm_dup" unsafe
	mpi_Comm_dup :: MPI_Comm -> Ptr MPI_Comm -> IO MPI_ReturnCode

foreign import ccall "MPI_Comm_compare" unsafe
	mpi_Comm_compare :: MPI_Comm -> MPI_Comm -> Ptr CInt 
                         -> IO MPI_ReturnCode

foreign import ccall "MPI_Comm_group" unsafe
	mpi_Comm_group :: MPI_Comm -> Ptr MPI_Group -> IO MPI_ReturnCode

-- should go to MPI_Group.hs ??
foreign import ccall "MPI_Barrier" unsafe
	mpi_Barrier :: MPI_Comm -> IO MPI_ReturnCode


-- Datatypes -----------------------------------------------------------------
foreign import ccall "MPI_Type_extent" unsafe
        mpi_Type_extent :: MPI_Datatype -> Ptr MPI_AInt
                        -> IO MPI_ReturnCode


#ifdef HAVE_MPI_GET_EXTENT
foreign import ccall "MPI_Type_get_extent" unsafe
        mpi_Type_get_extent :: MPI_Datatype -> Ptr MPI_AInt -> Ptr MPI_AInt
                            -> IO MPI_ReturnCode
#else
mpi_Type_get_extent :: MPI_Datatype -> Ptr MPI_AInt -> Ptr MPI_AInt
                    -> IO MPI_ReturnCode
mpi_Type_get_extent _ _ _ = unsupportedCall "MPI_Type_get_extent"
#endif

foreign import ccall "MPI_Type_size" unsafe
	mpi_Type_size :: MPI_Datatype -> Ptr CInt -> IO MPI_ReturnCode

#ifdef HAVE_MPI_TYPE_DUP
foreign import ccall "MPI_Type_dup" unsafe
	mpi_Type_dup :: MPI_Datatype -> Ptr MPI_Datatype -> IO MPI_ReturnCode
#else
mpi_Type_dup :: MPI_Datatype -> Ptr MPI_Datatype -> IO MPI_ReturnCode
mpi_Type_dup _ _ = unsupportedCall "MPI_Type_dup"
#endif


foreign import ccall "MPI_Type_contiguous" unsafe
	mpi_Type_contiguous :: CInt -> MPI_Datatype -> Ptr MPI_Datatype
                            -> IO MPI_ReturnCode

foreign import ccall "MPI_Type_vector" unsafe
	mpi_Type_vector :: CInt -> CInt -> CInt -> MPI_Datatype
                        -> Ptr MPI_Datatype -> IO MPI_ReturnCode

#ifdef HAVE_MPI_CREATE_HVECTOR
foreign import ccall "MPI_Type_create_hvector" unsafe
	mpi_Type_create_hvector :: CInt -> CInt -> CInt -> MPI_Datatype 
                                -> Ptr MPI_Datatype -> IO MPI_ReturnCode
#else
mpi_Type_create_hvector :: CInt -> CInt -> CInt -> MPI_Datatype 
                        -> Ptr MPI_Datatype -> IO MPI_ReturnCode
mpi_Type_create_hvector _ _ _ _ _ = unsupportedCall "MPI_Type_create_hvector"
#endif


-- p137

foreign import ccall "MPI_Type_commit" unsafe
	mpi_Type_commit :: Ptr MPI_Datatype -> IO MPI_ReturnCode

foreign import ccall "MPI_Type_free" unsafe
	mpi_Type_free :: Ptr MPI_Datatype -> IO MPI_ReturnCode


-- HMPI ccalls --------------------------------------------------------------
foreign import ccall unsafe
	hmpi_status_get_count :: MPI_Status -> MPI_Datatype -> Int

foreign import ccall unsafe
        hmpi_status_get_tag :: MPI_Status -> MPI_Tag

foreign import ccall unsafe
        hmpi_status_get_source :: MPI_Status -> MPI_Rank

foreign import ccall unsafe
        hmpi_status_get_error :: MPI_Status -> MPI_Error


-----------------------------------------------------------------------------
