{- $Id: MPI_Comm.hs,v 1.1.1.2 2001/02/28 04:48:03 michaelw Exp $
 - -*- Haskell -*-
 -----------------------------------------------------------------------------
 - MPI_Comm.hs - accessing MPI communicators
 -----------------------------------------------------------------------------
 -
 - hMPI - a Haskell binding for MPI 1.1/1.2
 - Copyright (C) 2000,2001 Michael Weber <michael.weber@post.rwth-aachen.de>
 -
 - This library is free software; you can redistribute it and/or
 - modify it under the terms of the GNU Lesser General Public License
 - as published by the Free Software Foundation; either version 2 of
 - the License, or (at your option) any later version.
 - 
 - This library is distributed in the hope that it will be useful, but
 - WITHOUT ANY WARRANTY; without even the implied warranty of
 - MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 - Lesser General Public License for more details.
 - 
 - You should have received a copy of the GNU Lesser General Public
 - License along with this library; if not, write to the Free Software
 - Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 - USA
 - 
 -----------------------------------------------------------------------------
 - NOTES:                                                                      
 - 
 -----------------------------------------------------------------------------
 - TODO:                                                                       
 - * document this file properly                                               
 -----------------------------------------------------------------------------
 -}

module MPI_Comm
    ( commGetRank
    , commGetSize
--     , commCreate
    , commFree
    , commDup
--     , commCompare
--     , commGroup
    , barrier

    , commMaxRank
    , maxRank

    , withComm
    )
where

import MPI_Types
import MPI_CCalls
import MPI_MarshalInternal
import Foreign

import IOExts (unsafePerformIO)
import Monad (liftM)


------------------------------------------------------------------------------
commGetRank :: Comm -> IO Rank
commGetRank comm = liftM unmarshalRank (do
    tempAllocOut_ (safeCall_ .
                  (mpi_Comm_rank (marshalComm comm)) . castPtr))


commGetSize :: Comm -> IO Int
commGetSize comm =
    tempAllocOut_ (safeCall_ .
                  (mpi_Comm_size (marshalComm comm)) . castPtr)


-- commCreate :: Comm -> Group -> IO Comm
-- commCreate comm group = liftM cToEnum $
--     (mpi_Comm_create (cFromEnum comm) (cFromEnum group) `marsh1` intAddr)
--         `ifMPIfailsM` "MPI_Comm_create"


commFree :: Comm -> IO ()
commFree comm = do
    tempAllocIn (marshalComm comm) 
                (safeCall_ .
                 mpi_Comm_free)


commDup :: Comm -> IO Comm
commDup comm = liftM unmarshalComm (do
    tempAllocOut_ (safeCall_ . 
                  (mpi_Comm_dup (marshalComm comm))))


-- commCompare :: Comm -> Comm -> IO Compare
-- commCompare c1 c2 = unmarshalCompare `safeCall` do
--     allocaElem cIntTy (\ comp -> do
--     r <- mpi_Comm_compare (marshalComm c1) (marshalComm c2) comp
--     bool <- peek comp
--     return (r,comp))


-- commGroup :: Comm -> IO Group
-- commGroup comm = liftM cToEnum $
--     (mpi_Comm_group (cFromEnum comm) `marsh1` intAddr)
--         `ifMPIfailsM` "MPI_Comm_group"


barrier :: Comm -> IO ()
barrier comm = 
    safeCall_ (mpi_Barrier (marshalComm comm))


{-# NOINLINE commMaxRank #-}
commMaxRank :: Comm -> Rank
commMaxRank comm = unsafePerformIO (do
    size <- commGetSize comm
    return (rank (size - 1)))


{-# NOINLINE maxRank #-}
maxRank :: Rank
maxRank = commMaxRank CommWorld


-- Helper stuff --------------------------------------------------------------
withComm :: IO Comm -> (Comm -> IO a) -> IO a
withComm allocComm p = do
    newcomm <- allocComm
    v <- p newcomm
    commFree newcomm
    return v


------------------------------------------------------------------------------
