{- $Id: MPI_Datatypes.hs,v 1.1.1.2 2001/02/28 04:48:03 michaelw Exp $
 - -*- Haskell -*-
 -----------------------------------------------------------------------------
 - MPI_Datatypes.hs - functions to manipulate MPI datatypes
 -----------------------------------------------------------------------------
 -
 - hMPI - a Haskell binding for MPI 1.1/1.2
 - Copyright (C) 2000,2001 Michael Weber <michael.weber@post.rwth-aachen.de>
 -
 - This library is free software; you can redistribute it and/or
 - modify it under the terms of the GNU Lesser General Public License
 - as published by the Free Software Foundation; either version 2 of
 - the License, or (at your option) any later version.
 - 
 - This library is distributed in the hope that it will be useful, but
 - WITHOUT ANY WARRANTY; without even the implied warranty of
 - MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 - Lesser General Public License for more details.
 - 
 - You should have received a copy of the GNU Lesser General Public
 - License along with this library; if not, write to the Free Software
 - Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 - USA
 - 
 -----------------------------------------------------------------------------
 - NOTES:                                                                      
 -
 -----------------------------------------------------------------------------
 - TODO:                                                                       
 - * document this file properly                                               
 -----------------------------------------------------------------------------
 -}

module MPI_Datatypes
    ( typeSize
    , typeGetExtent
    , typeExtent
    , typeDup
    , typeContiguous
    , typeVector
    , typeCreateHVector
    , typeCommit
    , typeFree
    )
where

import MPI_Types
import MPI_CCalls
import MPI_MarshalInternal
import Foreign.Ptr

import Monad


------------------------------------------------------------------------------
typeSize :: Datatype -> IO Int
typeSize dt = do
    tempAllocOut_ (safeCall_ . 
                  (mpi_Type_size (marshalDatatype dt)) . 
		   castPtr)


typeGetExtent :: Datatype -> IO (Integer,Integer)
typeGetExtent dt = do
    (lb,ext) <- tempAllocOut  (\ plb  -> do
                tempAllocOut_ (\ pext -> do
                safeCall_ (mpi_Type_get_extent (marshalDatatype dt) 
                                               plb pext)
                ))
    return (aIntToInteger lb, aIntToInteger ext)


typeExtent :: Datatype -> IO Integer
typeExtent dt = liftM aIntToInteger (do
    tempAllocOut_ (safeCall_ . 
                  (mpi_Type_extent (marshalDatatype dt))))


typeDup :: Datatype -> IO Datatype
typeDup dt = liftM unmarshalDatatype (do
    tempAllocOut_ (safeCall_ . 
                  (mpi_Type_dup (marshalDatatype dt))))


typeContiguous :: Int -> Datatype -> IO Datatype
typeContiguous count dt = liftM unmarshalDatatype (do
    tempAllocOut_ (safeCall_ . 
                  (mpi_Type_contiguous (fromIntegral count)
                                       (marshalDatatype dt))))


typeVector :: Int -> Int -> Int -> Datatype -> IO Datatype
typeVector count blklen stride dt = liftM unmarshalDatatype (do
    tempAllocOut_ (safeCall_ . 
                  (mpi_Type_vector (fromIntegral count)
                                   (fromIntegral blklen)
                                   (fromIntegral stride)
                                   (marshalDatatype dt))))


typeCreateHVector :: Int -> Int -> Int -> Datatype -> IO Datatype
typeCreateHVector count blklen stride dt = liftM unmarshalDatatype (do
    tempAllocOut_ (safeCall_ . 
                  (mpi_Type_create_hvector (fromIntegral count)
                                           (fromIntegral blklen)
                                           (fromIntegral stride)
                                           (marshalDatatype dt))))


typeCommit :: Datatype -> IO Datatype
typeCommit dt = liftM unmarshalDatatype (do
    tempAllocInout_ (marshalDatatype dt)
                    (safeCall_ . 
                     mpi_Type_commit))


typeFree :: Datatype -> IO ()
typeFree dt = do
    tempAllocIn (marshalDatatype dt) 
                (safeCall_ . 
                 mpi_Type_free)


------------------------------------------------------------------------------
