{- $Id: MPI_Group.hs,v 1.1.1.2 2001/02/28 04:48:03 michaelw Exp $
 - -*- Haskell -*-
 -----------------------------------------------------------------------------
 - MPI_Group.hs - manipulating MPI comm groups
 -----------------------------------------------------------------------------
 -
 - hMPI - a Haskell binding for MPI 1.1/1.2
 - Copyright (C) 2000,2001 Michael Weber <michael.weber@post.rwth-aachen.de>
 -
 -----------------------------------------------------------------------------
 - NOTES:                                                                      
 - 
 - This library is free software; you can redistribute it and/or
 - modify it under the terms of the GNU Lesser General Public License
 - as published by the Free Software Foundation; either version 2 of
 - the License, or (at your option) any later version.
 - 
 - This library is distributed in the hope that it will be useful, but
 - WITHOUT ANY WARRANTY; without even the implied warranty of
 - MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 - Lesser General Public License for more details.
 - 
 - You should have received a copy of the GNU Lesser General Public
 - License along with this library; if not, write to the Free Software
 - Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 - USA
 - 
 -----------------------------------------------------------------------------
 - TODO:                                                                       
 - * document this file properly                                               
 -----------------------------------------------------------------------------
 -}

module MPI_Group
    ( getGroupRank
    , getGroupSize
    , groupFree
    , groupUnion
    , groupIntersection
    , groupDifference
    , groupIncl
    , groupExcl
    , withGroup
    , groupCompare
    )
where

import MPI_Types
import MPI_CCalls
import MPI_MarshalInternal
import Foreign

import Monad


------------------------------------------------------------------------------
getGroupRank :: Group -> IO Rank
getGroupRank group = liftM unmarshalRank (do
    tempAllocOut_ (safeCall_ .
                  (mpi_Group_rank (marshalGroup group))))


getGroupSize :: Group -> IO Int
getGroupSize group = do
    tempAllocOut_ (safeCall_ .
                  (mpi_Group_size (marshalGroup group)) .
		   castPtr)


groupFree :: Group -> IO ()
groupFree group = do
    tempAllocIn (marshalGroup group) 
                (safeCall_ .
                 mpi_Group_free)


groupUnion :: Group -> Group -> IO Group
groupUnion = groupSetOp mpi_Group_union


groupIntersection :: Group -> Group -> IO Group
groupIntersection = groupSetOp mpi_Group_intersection


groupDifference :: Group -> Group -> IO Group
groupDifference = groupSetOp mpi_Group_difference


groupIncl :: Group -> [Rank] -> IO Group
groupIncl g ranks = do
     let len    = length ranks
         cRanks = map marshalRank ranks
     allocaElems (head cRanks) len (\ pRanks -> do
     liftM unmarshalGroup (do
     tempAllocOut_ (\ pgroup -> do
     mapM_ (uncurry (pokeElemOff pRanks)) (zip [0 ..] cRanks)
     safeCall_ (mpi_Group_incl (marshalGroup g) 
                               (fromIntegral len)
                               pRanks
                               pgroup))))


groupExcl :: Group -> [Rank] -> IO Group
groupExcl g ranks = do
     let len    = length ranks
         cRanks = map marshalRank ranks
     allocaElems (head cRanks) len (\ pRanks -> do
     liftM unmarshalGroup (do
     tempAllocOut_ (\ pgroup -> do
     mapM_ (uncurry (pokeElemOff pRanks)) (zip [0 ..] cRanks)
     safeCall_ (mpi_Group_excl (marshalGroup g) 
                               (fromIntegral len)
                               pRanks
                               pgroup))))


groupCompare :: Group -> Group -> IO Compare
groupCompare c1 c2 = liftM unmarshalCompare (do
    tempAllocOut_ (safeCall_ .
                  (mpi_Group_compare (marshalGroup c1)
                                     (marshalGroup c2))))


-- convenience functions -----------------------------------------------------
withGroup :: IO Group -> (Group -> IO a) -> IO a
withGroup g p = do
    newgroup <- g
    v <- p newgroup
    groupFree newgroup
    return v


-- Helper stuff --------------------------------------------------------------
groupSetOp :: (MPI_Group -> MPI_Group -> Ptr MPI_Group -> IO MPI_ReturnCode)
           -> Group
           -> Group
           -> IO Group
groupSetOp op g1 g2 = liftM unmarshalGroup (do
    tempAllocOut_ (safeCall_ . 
                  (op (marshalGroup g1) 
                      (marshalGroup g2))))


------------------------------------------------------------------------------
