{- $Id: MPI_Manage.hs,v 1.1.1.2 2001/02/28 04:48:03 michaelw Exp $
 - -*- Haskell -*-
 -----------------------------------------------------------------------------
 - MPI_Manage.hs - MPI management routines
 -----------------------------------------------------------------------------
 -
 - hMPI - a Haskell binding for MPI 1.1/1.2
 - Copyright (C) 2000,2001 Michael Weber <michael.weber@post.rwth-aachen.de>
 -
 - This library is free software; you can redistribute it and/or
 - modify it under the terms of the GNU Lesser General Public License
 - as published by the Free Software Foundation; either version 2 of
 - the License, or (at your option) any later version.
 - 
 - This library is distributed in the hope that it will be useful, but
 - WITHOUT ANY WARRANTY; without even the implied warranty of
 - MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 - Lesser General Public License for more details.
 - 
 - You should have received a copy of the GNU Lesser General Public
 - License along with this library; if not, write to the Free Software
 - Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 - USA
 - 
 -----------------------------------------------------------------------------
 - NOTES:                                                                      
 - * The type signature of the statusGet* functions may change in the
 -   future, if I discover that they belong into the IO Monad.
 -
 - * Should I use statusGetElements or statusGetCount? Is there a difference?
 -----------------------------------------------------------------------------
 - TODO:                                                                       
 - * document this file properly                                               
 -----------------------------------------------------------------------------
 -}

module MPI_Manage
    ( initialize
    , finalize
    , isInitialized
    , isFinalized

    , abort

    , getVersion
    , getProcessorName
    , getWTime
    , getWTick

    , withMPI

    , statusGetCount
    , statusGetTag
    , statusGetSource
    , statusGetError
    , statusGetElements
    )
where

import MPI_Types
import MPI_Consts
import MPI_CCalls
import MPI_Comm (barrier)
import MPI_MarshalInternal


import Foreign
import Monad


------------------------------------------------------------------------------
-- system control
initialize :: IO ()
initialize = safeCall_ hmpi_init


finalize :: IO ()
finalize = safeCall_ mpi_finalize


isInitialized :: IO Bool
isInitialized = liftM unmarshalBool (do
    tempAllocOut_ (safeCall_ . mpi_Initialized))


isFinalized :: IO Bool
isFinalized = liftM unmarshalBool (do
    tempAllocOut_ (safeCall_ . mpi_Finalized))


abort :: Comm -> Int -> IO () -- FIXME: SystemError?
abort comm ret = do
    let commC = marshalComm comm
        retC  = fromIntegral ret
    safeCall_ $ mpi_Abort commC retC


-- Info
getVersion :: IO Version
getVersion = liftM (uncurry newVersion) (do
    tempAllocOut  (\ maj -> do
    tempAllocOut_ (\ min -> do
    safeCall_ (mpi_Get_version (castPtr maj) (castPtr min))
    )))
		     

getProcessorName :: IO String
getProcessorName = do
    allocaElems charTy hmpi_max_processor_name $ \ pname -> do
    len  <- tempAllocOut_ (safeCall_ . 
                          (mpi_Get_processor_name pname)
			   . castPtr)
    unmarshalStringWithLen pname len


-- Timing
getWTime :: IO Double
getWTime = liftM unmarshalDouble mpi_Wtime


getWTick :: IO Double
getWTick = liftM unmarshalDouble mpi_Wtick


statusGetCount :: Status -> Datatype -> Maybe Int
statusGetCount status dt = 
    let 
      count = hmpi_status_get_count (statusPtr status)
                                    (marshalDatatype dt)
    in
      if count == mpiUndefined
        then Nothing 
        else Just count


statusGetTag :: Status -> Tag
statusGetTag status =
    unmarshalTag (hmpi_status_get_tag (statusPtr status))


statusGetSource :: Status -> Rank
statusGetSource status = 
    unmarshalRank (hmpi_status_get_source (statusPtr status))


statusGetError :: Status -> Error
statusGetError status = 
     newError (hmpi_status_get_error (statusPtr status))


statusGetElements :: Status -> Datatype -> IO Int
statusGetElements status dt = do
    tempAllocOut_ (safeCall_ .
                  (mpi_Get_elements (statusPtr status) 
                                    (marshalDatatype dt)) . castPtr)



-- convenience functions ------------------------------------------------------
withMPI :: IO a -> IO a
withMPI p = do
    initialize
    v <- p
    barrier CommWorld -- this is recommended, otherwise the behaviour of the
                      -- `I'-functions is undefined (IIRC)
    finalize
    return v


------------------------------------------------------------------------------
