{- $Id: MPI_MarshalInternal.hs,v 1.1.1.2 2001/02/28 04:48:03 michaelw Exp $
 - -*- Haskell -*-
 -----------------------------------------------------------------------------
 - MPI_MarshalInternal.hs - low level marshaling routines
 -----------------------------------------------------------------------------
 -
 - hMPI - a Haskell binding for MPI 1.1/1.2
 - Copyright (C) 2000,2001 Michael Weber <michael.weber@post.rwth-aachen.de>
 -
 - This library is free software; you can redistribute it and/or
 - modify it under the terms of the GNU Lesser General Public License
 - as published by the Free Software Foundation; either version 2 of
 - the License, or (at your option) any later version.
 - 
 - This library is distributed in the hope that it will be useful, but
 - WITHOUT ANY WARRANTY; without even the implied warranty of
 - MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 - Lesser General Public License for more details.
 - 
 - You should have received a copy of the GNU Lesser General Public
 - License along with this library; if not, write to the Free Software
 - Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 - USA
 - 
 -----------------------------------------------------------------------------
 - NOTES:
 -
 -----------------------------------------------------------------------------
 - TODO:                                                                       
 - * provide more marshaling primitives
 -
 - * document this file properly                                               
 -----------------------------------------------------------------------------
 -}

module MPI_MarshalInternal
--    ( module MPI_MarshalInternal
--    , module Foreign.
--    , module CTypes
--    )
where

import Foreign.Storable
import Foreign.Ptr
import Foreign
import CTypes

import Exception
import Dynamic
import Monad

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif



-----------------------------------------------------------------------------
type MPI_Rank_Type = Int
type MPI_Tag_Type  = Int

newtype MPI_Rank = MPI_Rank MPI_Rank_Type deriving (Eq)
newtype MPI_Tag  = MPI_Tag  MPI_Tag_Type  deriving (Eq)

#ifdef HAVE_LAM_MPI
#define HS_TYPE_GROUP		(Ptr Int)
#define HS_TYPE_COMM		(Ptr Int)
#define HS_TYPE_DATATYPE	(Ptr Int) 
#define HS_TYPE_OP		(Ptr Int)
#else 
#ifdef HAVE_MPICH_MPI
#define HS_TYPE_GROUP		CInt
#define HS_TYPE_COMM		CInt
#define HS_TYPE_DATATYPE	CInt 
#define HS_TYPE_OP		CInt 
#else
#error "unknown MPI implementation"
#endif
#endif


newtype MPI_Buffer a = MPI_Buffer (Ptr a) deriving (Eq)
newtype MPI_Request  = MPI_Request (ForeignPtr Int)  deriving (Eq)
newtype MPI_Status   = MPI_Status (ForeignPtr Int)  deriving (Eq)
newtype MPI_Group    = MPI_Group HS_TYPE_GROUP deriving (Eq)
newtype MPI_Op       = MPI_Op HS_TYPE_OP deriving (Eq)
newtype MPI_Comm     = MPI_Comm HS_TYPE_COMM deriving (Eq)
newtype MPI_Datatype = MPI_Datatype HS_TYPE_DATATYPE deriving (Eq)

type MPI_AInt        = CLong
type MPI_Error       = CInt
type MPI_ReturnCode  = CInt

instance Enum MPI_Rank where
    toEnum x              = MPI_Rank x
    fromEnum (MPI_Rank x) = x

instance Enum MPI_Tag where
    toEnum x             = MPI_Tag x
    fromEnum (MPI_Tag x) = x

instance Read MPI_Rank where
    readsPrec p s = map (\ (r,s) -> (MPI_Rank r,s)) (readsPrec p s)

instance Show MPI_Rank where
    showsPrec p (MPI_Rank r) = showsPrec p r

instance Storable MPI_Rank where
    sizeOf    (MPI_Rank r) = sizeOf r
    alignment (MPI_Rank r) = alignment r
    peek                   = liftM MPI_Rank . peek . castPtr
    poke addr (MPI_Rank r) = poke (castPtr addr) r


instance Read MPI_Tag where
    readsPrec p s = map (\ (t,s) -> (MPI_Tag t,s)) (readsPrec p s)

instance Show MPI_Tag where
    showsPrec p (MPI_Tag t) = showsPrec p t


instance Storable MPI_Comm where
    sizeOf    (MPI_Comm c) = sizeOf c
    alignment (MPI_Comm c) = alignment c

    peek                   = liftM MPI_Comm . peek . castPtr
    poke addr (MPI_Comm c) = poke (castPtr addr) c


instance Storable MPI_Group where
    sizeOf    (MPI_Group g) = sizeOf g
    alignment (MPI_Group g) = alignment g

    peek                    = liftM MPI_Group . peek . castPtr
    poke addr (MPI_Group g) = poke (castPtr addr) g


instance Show MPI_Datatype where
    showsPrec p (MPI_Datatype r) = showsPrec p r

instance Storable MPI_Datatype where
    sizeOf    (MPI_Datatype d) = sizeOf d
    alignment (MPI_Datatype d) = alignment d

    peek                       = liftM MPI_Datatype . peek . castPtr
    poke addr (MPI_Datatype d) = poke (castPtr addr) d


-----------------------------------------------------------------------------
data UnsupportedException =
    UnsupportedCall String

unsupportedExceptionTc = mkTyCon "UnsupportedException"

instance Typeable UnsupportedException where
  typeOf _ = mkAppTy unsupportedExceptionTc []


unsupportedCall :: String -> IO a
unsupportedCall s = throwDyn (UnsupportedCall s)
    

-----------------------------------------------------------------------------
charTy :: Char
charTy = undefined

cIntTy :: CInt
cIntTy = undefined

addrTy :: Ptr Int
addrTy = undefined

cLongTy :: CLong
cLongTy = undefined

mpiAIntTy :: MPI_AInt
mpiAIntTy = undefined


aIntToInteger :: MPI_AInt -> Integer
aIntToInteger = fromIntegral


-----------------------------------------------------------------------------
unmarshalBool :: CInt -> Bool
unmarshalBool 0 = False
unmarshalBool _ = True


unmarshalDouble :: CDouble -> Double
unmarshalDouble = fromRational . toRational


unmarshalStringWithLen ptr len =
  mapM (peekElemOff ptr) [0 .. (len - 1)]


-----------------------------------------------------------------------------
