{- $Id: MPI_SerializeDefault.hs,v 1.1.1.2 2001/02/28 04:48:03 michaelw Exp $
 - -*- Haskell -*-
 -----------------------------------------------------------------------------
 - MPI_SerializeDefault.hs - default data marshalling for Haskell types
 -----------------------------------------------------------------------------
 -
 - hMPI - a Haskell binding for MPI 1.1/1.2
 - Copyright (C) 2000,2001 Michael Weber <michael.weber@post.rwth-aachen.de>
 -
 - This library is free software; you can redistribute it and/or
 - modify it under the terms of the GNU Lesser General Public License
 - as published by the Free Software Foundation; either version 2 of
 - the License, or (at your option) any later version.
 - 
 - This library is distributed in the hope that it will be useful, but
 - WITHOUT ANY WARRANTY; without even the implied warranty of
 - MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 - Lesser General Public License for more details.
 - 
 - You should have received a copy of the GNU Lesser General Public
 - License along with this library; if not, write to the Free Software
 - Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 - USA
 - 
 -----------------------------------------------------------------------------
 - NOTES:                                                                      
 -
 -----------------------------------------------------------------------------
 - TODO:                                                                       
 -
 -----------------------------------------------------------------------------
 -}

module MPI_SerializeDefault
    ( module MPI_Serialize
    , module MPI_SerializeDefault
    )
where

import MPI_Serialize
import qualified MPI_Types as MPI
import MPI_MarshalInternal -- for CTypes
import IOExts
import Foreign
import Foreign.C
import Monad

-----------------------------------------------------------------------------
-- generic instances
{-instance (Serialize a,Storable a) => Serialize [a] where
    datatypeOf ls  = datatypeOf (head ls)
    sizeInElems    = length

    sizeInBytes ls = (sizeInElems ls) * (sizeInBytes (head ls))
    serialize      = liftM MPI.castBuffer . serializeList
    deserialize  i b  = deserializeList i (MPI.castBuffer b)
-}

-- this usage of the preprocessor saves quite some key strokes... =)
#define INSTANCE_SERIALIZE(T,MT) \
instance Serialize (T) where \
    { datatypeOf _  = MT \
    ; sizeInBytes   = sizeOf \
    ; serialize     = serializeElem \
    ; deserialize 0 = deserializeZeroError \
    ; deserialize _ = deserializeElem \
    }


-- Haskell types
INSTANCE_SERIALIZE(Char,    MPI.DataUnsignedChar)
INSTANCE_SERIALIZE(Int,     MPI.DataInt)
INSTANCE_SERIALIZE(Float,   MPI.DataFloat)
INSTANCE_SERIALIZE(Double,  MPI.DataDouble)

-- CTypes stuff
INSTANCE_SERIALIZE(CChar,   MPI.DataChar)
INSTANCE_SERIALIZE(CSChar,  MPI.DataChar)
INSTANCE_SERIALIZE(CUChar,  MPI.DataUnsignedChar)
INSTANCE_SERIALIZE(CShort,  MPI.DataShort)
INSTANCE_SERIALIZE(CUShort, MPI.DataUnsignedShort)
INSTANCE_SERIALIZE(CInt,    MPI.DataInt)
INSTANCE_SERIALIZE(CUInt,   MPI.DataUnsigned)
INSTANCE_SERIALIZE(CLong,   MPI.DataLong)
INSTANCE_SERIALIZE(CULong,  MPI.DataUnsignedLong)
INSTANCE_SERIALIZE(CFloat,  MPI.DataFloat)
INSTANCE_SERIALIZE(CDouble, MPI.DataDouble)


-- ZStrings provided by MPI_Serialize
instance Serialize ZString where
    datatypeOf _    = MPI.DataChar
    sizeInElems     = (1+) . length . fromZString

    sizeInBytes str = (1 + length (fromZString str)) 
                    * (sizeOf (head (fromZString str)))
    serialize       = serializeZString
    deserialize 0   = deserializeZeroError
    deserialize _   = deserializeZString


-----------------------------------------------------------------------------
deserializeZeroError _ = ioError (userError "MPI.deserialize: count = 0")
-- better? deserializeZeroError _ = return undefined 
-- better? deserializeZeroError _ = throwDyn ...


------------------------------------------------------------------------------
